%% Raketin tilayhtl ja linearisointi
%
% Saat generoitua tst MATLAB-skriptista documentin 
% valitsemalla File - Publish To - Word Document.
%
% Mallin tilayhtlt (sytetty latex-muodossa):
%
% $$v(t) = \dot{x}_1(t) = -\frac{H e^{\beta x_3(t)}}{m_0(t)+x_2(t)} \cdot x_1(t) \cdot |x_1(t)| + \frac{1}{m_0+x_2(t)} u(t) - g$$
%
% $$m(t) = \dot{x}_2(t) = -\frac{1}{c} u(t)$$
%
% $$h(t) = \dot{x}_3(t) = x_1(t)$$
%

%% Parametrien asetus
% Asetetaan simulink-mallien kyttmien parametrien arvot.

clear
g = 9.8;        % Putoamiskiihtyvyys
m0 = 1000;      % Raketin massa ilman polttoainetta
m_start = 500;  % Raketin polttoaineen massa alussa
beta = 0.1;
c = 1000;
H = 2000;
v0 = 1;         % Nopeus alussa 
h0 = 1;         % Korkeus alussa
timespan = [0 15]; %Simuloinnin alku- ja loppuaika

%% Eplineaarinen malli vakio-ohjauksellla
% Ohjauksen suuruus pidetn vakiona

smodel = 'lh2t4_A';
opts = simset(simget(smodel),'SaveFormat','Structure');
[time, ~, y] = sim(smodel, timespan, opts);
clf
for ii=1:4
    subplot(2,2,ii)
    plot(time, y.signals(ii).values);
    title(y.signals(ii).blockName, 'interpreter', 'none') 
    % Interpreter none: alaviivaa ei tulkita alaviitteeksi
end

%% Eplineaarinen malli graavitaation mukaisella sdll
%  Voimaa sdettess otettu huomioon gravitaatio:
%  nopeus lhestyy nollaa ja korkeus vakioarvoa.

smodel = 'lh2t4_B';
opts = simset(simget(smodel),'SaveFormat','Structure');
[time, ~, y] = sim(smodel, timespan, opts);
clf
for ii=1:4
    subplot(2,2,ii)
    plot(time, y.signals(ii).values);
    title(y.signals(ii).blockName, 'interpreter', 'none') 
end

%% Eplineaarinen malli vakionopeusohjauksella
%  Voimaa sdettess otettu huomioon sek gravitaatio
%  ett ilmanvastus, eli nopeus saadaan pysymn vakiona.

smodel = 'lh2t4_C';
opts = simset(simget(smodel),'SaveFormat','Structure');
[time, ~, y] = sim(smodel, timespan, opts);
clf
for ii=1:4
    subplot(2,2,ii)
    plot(time, y.signals(ii).values);
    title(y.signals(ii).blockName, 'interpreter', 'none') 
end

%% Linearisoitu malli, testi A
% Ajetaan eplineaarinen malli vakionopeusohjauksella

smodel = 'lh2t4_tallentava';
% Aika jolloin vaihdetaan vakionopeusohjaukseen
time_to_1 = -1;     %#ok 
% Tyntvoiman nollausaika
time_to_2 = 1000;   %#ok
opts = simset(simget(smodel),'SaveFormat','Structure');
[time, ~, y] = sim(smodel, timespan, opts); %#ok
% Malli tallentaa muuttuja-arvot ja derivaatat 
% MATLAB workspaceen.
% HUOM! Kommentti %#ok rivin perss poistaa m-lint-varoituksen 
% editorin oikeasta laidasta

% Ajetaan linearisoitu malli
clf
delta_time = 5; %#ok
delta_u = 0;    %#ok
smodel = 'lh2t4_linearisoitu';
opts = simset(simget(smodel),'SaveFormat','Structure');
[timel, ~, yl] = sim(smodel, timespan, opts);
% Malli lukee muuttuja-arvot ja derivaatat
% MATLAB workspacesta

% Visualisoidaan linearisoidun mallin ulostulo
for ii=1:3
    subplot(2,2,ii)
    plot(timel, yl.signals(ii).values, 'r.');
    title(yl.signals(ii).blockName, 'interpreter', 'none') 
end
% Visualisoidaan tyntvoima
subplot(2,2,4)
plot(timel, yl.signals(4).values, '.-');
title(yl.signals(4).blockName, 'interpreter', 'none')
% Verrataan eplineaarisen mallin kyttytymiseen samalla ohjauksella
for ii=5:7
    subplot(2,2,ii-4)
    hold on;
    plot(timel, yl.signals(ii).values);
end

%% Linearisoitu malli, testi B
% Ajetaan eplineaarinen malli vaihtelevalla ohjauksella:
%
% * Ensin voima pidetn vakiona;
% * Sitten sdetn ohjausta siten ett nopeus pysyy vakiona;
% * Lopuksi voima asetetaan nollaan.

smodel = 'lh2t4_tallentava';
time_to_1 = 5;  %#ok
time_to_2 = 10; %#ok
opts = simset(simget(smodel),'SaveFormat','Structure');
[time, x, y] = sim(smodel, timespan, opts); %#ok

% Ajetaan linearisoitu malli
clf
delta_time = 5; %#ok
delta_u = 0;    %#ok
smodel = 'lh2t4_linearisoitu';
opts = simset(simget(smodel),'SaveFormat','Structure');
[timel, ~, yl] = sim(smodel, timespan, opts);

for ii=1:3
    subplot(2,2,ii)
    plot(timel, yl.signals(ii).values, 'r.');
    title(yl.signals(ii).blockName, 'interpreter', 'none') 
end
subplot(2,2,4)
plot(timel, yl.signals(4).values, '.-');
title(yl.signals(4).blockName, 'interpreter', 'none')
for ii=5:7
    subplot(2,2,ii-4)
    hold on;
    plot(timel, yl.signals(ii).values);
end

%% Linearisoitu malli, testi C
% Haetaan tasapainopisteet ajemalla eplineaarinen malli 
% vakionopeus-ohjauksella: poikkeutetaan ohjausta 5 sekunnin kohdalla 
% lineaarista mallia ajettaessa

smodel = 'lh2t4_tallentava';
time_to_1 = -1;
time_to_2 = 1000;
opts = simset(simget(smodel),'SaveFormat','Structure');
[time, x, y] = sim(smodel, timespan, opts);

% Ajetaan linearisoitu malli
clf
delta_time = 5;
delta_u = 5000;
smodel = 'lh2t4_linearisoitu';
opts = simset(simget(smodel),'SaveFormat','Structure');
[timel, xl, yl] = sim(smodel, timespan, opts);

for ii=1:3
    subplot(2,2,ii)
    plot(timel, yl.signals(ii).values, 'r.');
    title(yl.signals(ii).blockName, 'interpreter', 'none') 
end
subplot(2,2,4)
plot(timel, yl.signals(4).values, '.-');
title(yl.signals(4).blockName, 'interpreter', 'none')
for ii=5:7
    subplot(2,2,ii-4)
    hold on;
    plot(timel, yl.signals(ii).values);
end

